import pyglet
from pyglet.gl import *
from pyglet.window import key
import cocos
from cocos.menu import *
from cocos.layer import *
from cocos.particle_systems import Sun, Smoke
from simulation import *
import math
import random
import itemplacement

TILE_SIZE = 60

class MainGameViewLayer(Layer):
    is_event_handler = True # enable pyglet's events

    def __init__(self, gameState):
        super( MainGameViewLayer, self ).__init__()
        self.gameState = gameState
        self.schedule_interval(self.update, 0.1)
        self.add(BackgroundLayer(gameState.playfield),z=-1)
        self.add(TerrainLayer(gameState.playfield),z=0)
        self.add(ActorLayer(gameState),z=1)
        self.ItemPlacementScene = cocos.scene.Scene (itemplacement.ItemChooser(gameState))
        
    def update(self, dt):
        '''
        this is called every 0.1 seconds (or whatever is set in the schedule_interval call in __init__
        dt is the time since the last call, in seconds.
        '''
        self.gameState.update(dt)

    def on_key_press (self, symbol, modifiers):
        """This function is called when a key is pressed.
        'key' is a constant indicating which key was pressed.
        'modifiers' is a bitwise or of several constants indicating which
            modifiers are active at the time of the press (ctrl, shift, capslock, etc.)
        """
        if symbol == key.UP:
            self.gameState.OldMan.move_up()
        elif symbol == key.DOWN:
            self.gameState.OldMan.move_down()
        elif symbol == key.RIGHT:
            self.gameState.OldMan.move_right()
        elif symbol == key.LEFT:
            self.gameState.OldMan.move_left()
        elif symbol == key.ENTER or symbol == key.NUM_ENTER:
            cocos.director.director.push(self.ItemPlacementScene)

    def on_key_release (self, key, modifiers):
        """This function is called when a key is released.

        'key' is a constant indicating which key was pressed.
        'modifiers' is a bitwise or of several constants indicating which
            modifiers are active at the time of the press (ctrl, shift, capslock, etc.)

        Constants are the ones from pyglet.window.key
        """
        pass
    
    def draw(self):
        super(MainGameViewLayer, self).draw()
        for child in self.get_children():
            child.draw()

class OldMan(cocos.cocosnode.CocosNode):
    def __init__(self, oldmanSimulationObject):
        super(OldMan, self).__init__()
        self.sim = oldmanSimulationObject
        self.smoke = Smoke()
        self.smoke.angle = 45
        self.smoke.speed = 10
        self.smoke.speed_var = 5
        self.fire = Sun()
        self.add(self.smoke)
        self.add(self.fire)
        self.schedule_interval(self.update_smoke, 0.2)
    def update_smoke(self,dt):
        self.smoke.angle += random.randrange(-5,5)
    def draw(self):
        location = self.sim.get_location()
        self.smoke.position = (location.x*TILE_SIZE+TILE_SIZE/2, location.y*TILE_SIZE+TILE_SIZE/3)
        self.fire.position = (location.x*TILE_SIZE+TILE_SIZE/2, location.y*TILE_SIZE+TILE_SIZE/3)        

class ActorLayer(Layer):
    def __init__(self, gameState):
        super(ActorLayer, self).__init__()
        self.gameState = gameState
        oldman = OldMan(gameState.OldMan)
        self.add(oldman)
        
    def draw(self):
        super(ActorLayer,self).draw()
        for child in self.get_children():
            child.draw()
        if self.gameState.person is not None:
            self.draw_person(self.gameState.person)
            
    def draw_person(self,person):
        x = person.get_location().x*TILE_SIZE + TILE_SIZE/2
        y = person.get_location().y*TILE_SIZE + TILE_SIZE/2
        self.draw_circle(x,y)
        
    def draw_circle(self, x, y):
        size = TILE_SIZE/4
        segments = 8
        glColor4f(1,0.5,1,1)
        glPushMatrix()
        glTranslatef(x-size/2, y-size/2,0)
        glBegin(GL_POLYGON)
        for i in range(16):
            glVertex2f(size*math.sin(i*math.pi/segments), size*math.cos(i*math.pi/segments))
        glEnd()
        glPopMatrix()
        gl.glColor4f(1,1,1,1)         

class TerrainLayer(Layer):
    def __init__(self, playfield):
        super(TerrainLayer, self).__init__()
        self.playfield = playfield
        
    def draw(self):
        super(TerrainLayer,self).draw()
        for cell in self.playfield.field.values():
            for obj in cell.get_objects():
                obj_type = obj.get_type()
                if obj_type not in (PF_HOUSE, PF_WINDOW, PF_DOOR, PF_SIDEWALK, PF_SHED, PF_OLD_MAN, PF_PERSON):
                    #it's a terrain item
                    self.draw_terrain(obj, cell.x, cell.y)
            if self.playfield.has_fence( ((cell.x, cell.y), (cell.x, cell.y-1)) ):
                self.draw_horizontal_fence(cell.x,cell.y)
            if self.playfield.has_fence( ((cell.x, cell.y), (cell.x-1, cell.y)) ):
                self.draw_vertical_fence(cell.x,cell.y)
            
    def draw_horizontal_fence(self, x, y):
        glColor4f(0.5,0.5,0.3,1)
        x, y = x*TILE_SIZE, y*TILE_SIZE
        w = x+TILE_SIZE; h=y+TILE_SIZE/16
        glBegin(gl.GL_QUADS)
        glVertex2f( x, y )
        glVertex2f( x, h )
        glVertex2f( w, h )
        glVertex2f( w, y )
        glEnd()
        glColor4f(1,1,1,1) 
    
    def draw_vertical_fence(self, x, y):
        glColor4f(0.5,0.5,0.3,1)
        x, y = x*TILE_SIZE, y*TILE_SIZE
        w = x+TILE_SIZE/16; h=y+TILE_SIZE
        glBegin(gl.GL_QUADS)
        glVertex2f( x, y )
        glVertex2f( x, h )
        glVertex2f( w, h )
        glVertex2f( w, y )
        glEnd()
        glColor4f(1,1,1,1) 

    def draw_terrain(self, obj, x, y):
        if obj.get_type() == PF_FLOWERBED:
            color = ((0.5 * obj.bloom), 0, (0.5 * obj.bloom), 1)
        elif obj.get_type() == PF_BUSH:
            color = (0, 1, 0, 1)
        else:
            color = (0.5, 0.5, 1,1)
        glColor4f(*color)
        x, y = x*TILE_SIZE+TILE_SIZE/4, y*TILE_SIZE+TILE_SIZE/4
        w = x+TILE_SIZE/2; h=y+TILE_SIZE/2
        glBegin(gl.GL_QUADS)
        glVertex2f( x, y )
        glVertex2f( x, h )
        glVertex2f( w, h )
        glVertex2f( w, y )
        glEnd()
        glColor4f(1,1,1,1) 
    
class BackgroundLayer(Layer):
    def __init__(self, playfield):
        super(BackgroundLayer, self).__init__()
        for cell in playfield.field.values():
            for obj in cell.get_objects():
                obj_type = obj.get_type()
                if obj_type in (PF_HOUSE, PF_WINDOW, PF_DOOR, PF_SIDEWALK, PF_SHED):
                    #it's part of the bg
                    self.add(Tile(obj_type, cell.x, cell.y))
        # just use one big ass grass tile underneath
        self.add(Tile(PF_EMPTY,0,0, size=TILE_SIZE*10))
    
    def draw(self):
        super(BackgroundLayer, self).draw()
        for child in self.get_children():
            child.draw()
                    
class Tile(cocos.cocosnode.CocosNode):
    """
    Tile (color, c, y, size=50) : A layer drawing a Tile at (x,y) of
    given tileType
    """
    def __init__(self, tileType , x, y, size=TILE_SIZE):
        super( Tile, self ).__init__()
        self.x = x*size
        self.y = y*size
        self.size = size
        self.layer_color = self.type_to_color(tileType)
                
    def draw(self):
        super(Tile ,self).draw()
        gl.glColor4f(*self.layer_color)
        x, y = self.x, self.y
        w = x+self.size; h=y+self.size
        gl.glBegin(gl.GL_QUADS)
        gl.glVertex2f( x, y )
        gl.glVertex2f( x, h )
        gl.glVertex2f( w, h )
        gl.glVertex2f( w, y )
        gl.glEnd()
        gl.glColor4f(1,1,1,1) 

    def type_to_color(self, tileType):
        if tileType == PF_EMPTY:
            color = (0,0.4, 0,1)
        elif tileType == PF_HOUSE:
            color = (0.5, 0.5, 0, 1)
        elif tileType == PF_WINDOW:
            color = (0,0,0.5,1)
        elif tileType == PF_DOOR:
            color = (0.1,0.1,0,1)
        elif tileType == PF_SIDEWALK:
            color = (0.8,0.8,0.8,1)
        elif tileType == PF_SHED:
            color = (0.8,0.6,0.4,1)
        
        return color
